package org.blue.club.configuration;

import lombok.RequiredArgsConstructor;
import org.blue.club.annotation.Auth;
import org.blue.club.dao.UserDao;
import org.blue.club.entities.dto.User;
import org.blue.club.entities.vo.UserVo;
import org.mmga.spring.boot.starter.componet.AuthorizationHandler;
import org.mmga.spring.boot.starter.componet.JwtUtils;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.exception.AuthorizationException;
import org.mmga.spring.boot.starter.utils.VoUtils;
import org.springframework.stereotype.Component;

import java.lang.annotation.Annotation;
import java.util.Optional;

@Component
@RequiredArgsConstructor
public class CustomAuthorizationHandler implements AuthorizationHandler<User> {
    private final JwtUtils jwtUtils;
    private final UserDao userDao;
    private final VoUtils voUtils;

    @Override
    public Optional<User> auth(String token, Annotation ann) {
        if (ann instanceof Auth auth) {
            Optional<Long> i = jwtUtils.verifyToken(token);
            if (i.isEmpty()) throw new AuthorizationException(Result.failed("token错误！"));
            Long userId = i.get();
            UserVo userVo = userDao.selectOneWithRelationsById(userId);
            if (userVo == null) throw new AuthorizationException(Result.failed("用户不存在！"));
            User user = voUtils.vo2DtoSafe(userVo, User.class);

            return Optional.ofNullable(user);
        }
        return Optional.empty();
    }
}
