package org.blue.club.services;

import lombok.RequiredArgsConstructor;
import org.blue.club.dao.UserDao;
import org.blue.club.dao.redis.VerifyDao;
import org.blue.club.entities.dto.User;
import org.blue.club.entities.dto.req.LoginRequest;
import org.blue.club.entities.dto.req.RegisterRequest;
import org.blue.club.entities.vo.UserVo;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.utils.VoUtils;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.Optional;

import static org.blue.club.entities.vo.table.UserVoTableDef.USER_VO;

@Service
@RequiredArgsConstructor
public class UserServices {
    private final UserDao userDao;
    private final VerifyDao verifyDao;
    private final VoUtils voUtils;

    private boolean isWrongVerifyCode(String key, String code) {
        Optional<String> targetCode = verifyDao.findById(key);
        if (targetCode.isEmpty()) {
            return true;
        }
        verifyDao.deleteById(key);
        return !targetCode.get().equals(code);
    }

    public Result<User> login(LoginRequest loginRequest) {
        if (this.isWrongVerifyCode(loginRequest.key(), loginRequest.code())) {
            return Result.failed(HttpStatus.FORBIDDEN, "错误的验证码");
        }
        UserVo userVo = userDao.selectOneWithRelationsByCondition(USER_VO.NAME.eq(loginRequest.username()).and(USER_VO.PASSWORD.eq(loginRequest.sha1HexPassword())));
        if (userVo == null) return Result.failed(HttpStatus.NOT_FOUND, "用户不存在！");
        User user = voUtils.vo2DtoSafe(userVo, User.class);
        return Result.success(user);
    }

    public Result<User> createUser(RegisterRequest request) {
        if (this.isWrongVerifyCode(request.key(), request.code())) {
            return Result.failed(HttpStatus.FORBIDDEN, "错误的验证码");
        }
        if (userDao.selectCountByCondition(USER_VO.NAME.eq(request.username())) > 0) {
            return Result.failed(HttpStatus.CONFLICT, "用户已存在");
        }
        userDao.insert()
        return null;
    }
}
